import{p as C,a as g,s as k,b as E,o as J,c,j as w,z as R,l as v,y as q,i as b,g as a,t as B,k as F,e as m,J as G,n as H,d as K,f as L,A as N,h as P,D as Q}from"./index-BtH6urND.js";import T from"./Markdown-BDB-IX6n.js";import"./purify.es-HRjpPm7y.js";import"./katex-XbL3y5x-.js";/* empty css                  */var U=w('<details class="svelte-1b7h7k8"><summary class="svelte-1b7h7k8"> </summary> <div class="panel svelte-1b7h7k8"><!></div></details>'),V=w('<div class="accordion svelte-1b7h7k8"></div>');function ee(y,i){C(i,!0);let h=g(i,"items",19,()=>[]),p=g(i,"allowMultiple",3,!1),r=k(null),n=k(E(new Set));J(()=>{const t=new Set;if(h().forEach((e,l)=>{e.defaultOpen&&t.add(l)}),p())c(n,t,!0);else{const[e]=Array.from(t);c(r,e??null,!0)}});function M(t){if(p()){const e=new Set(a(n));e.has(t)?e.delete(t):e.add(t),c(n,e,!0);return}c(r,a(r)===t?null:t,!0)}function A(t){return p()?a(n).has(t):a(r)===t}var _=V();R(_,21,h,N,(t,e,l)=>{var u=U(),d=v(u);d.__click=s=>{s.preventDefault(),M(l)};var S=v(d),D=q(d,2),I=v(D);{var O=s=>{{let o=H(()=>a(e).props??{});G(s,{get component(){return a(e).component},get props(){return a(o)}})}},j=s=>{var o=K(),x=L(o);{var z=f=>{T(f,{get content(){return a(e).content},padded:!1})};b(x,f=>{a(e).content&&f(z)},!0)}m(s,o)};b(I,s=>{a(e).component?s(O):s(j,!1)})}B(s=>{u.open=s,F(S,a(e).title)},[()=>A(l)]),m(t,u)}),m(y,_),P()}Q(["click"]);export{ee as default};
