import{a as t,j as h,t as m,w as d,l as w,r as p,e as y}from"./index-BtH6urND.js";var _=h('<div class="video-wrapper svelte-1737ekv"><video class="svelte-1737ekv"></video></div>',2);function k(l,e){let s=t(e,"src",3,""),i=t(e,"poster",3,void 0),r=t(e,"autoplay",3,!1),u=t(e,"controls",3,!0),v=t(e,"loop",3,!1),f=t(e,"muted",3,!1),n=t(e,"width",3,void 0),c=t(e,"height",3,void 0);var o=_(),a=w(o);m(()=>{d(a,"src",s()),d(a,"poster",i()),a.autoplay=r(),a.controls=u(),a.loop=v(),a.muted=f(),p(a,`width: ${n()||"100%"}; height: ${c()||"auto"};`)}),y(l,o)}export{k as default};
