import{s as r,b as e,a,S as s}from"./chunk-DI55MBZ5-D24KjRxa.js";import{_ as i}from"./Mermaid-Csfl9UYS.js";import"./chunk-55IACEB6-BtkK7AIg.js";import"./chunk-QN33PNHL-DcFSm8MN.js";import"./index-BtH6urND.js";import"./_commonjsHelpers-CqkleIqs.js";import"./purify.es-HRjpPm7y.js";var n={parser:a,get db(){return new s(2)},renderer:e,styles:r,init:i(t=>{t.state||(t.state={}),t.state.arrowMarkerAbsolute=t.arrowMarkerAbsolute},"init")};export{n as diagram};
