.mcp-app-frame.svelte-134bwg1{width:100%;border:none;background:var(--color-card-bg, #fcfaf7);border-radius:calc(var(--radius-lg, 12px) - 2px)}.mcp-app-frame.bordered.svelte-134bwg1{border:1px solid var(--border-color, #e2e8f0)}.mcp-app-loading.svelte-134bwg1{display:flex;align-items:center;justify-content:center;height:200px;color:var(--text-muted, #6b7280);font-size:.875rem}.mcp-app-error.svelte-134bwg1{display:flex;align-items:center;justify-content:center;height:200px;color:var(--text-error, #ef4444);font-size:.875rem}
