import{s as a,c as s,a as e,C as t}from"./chunk-B4BG7PRW-wfpmNHz8.js";import{_ as i}from"./Mermaid-Csfl9UYS.js";import"./chunk-FMBD7UC4-DgQ8ie6r.js";import"./chunk-55IACEB6-BtkK7AIg.js";import"./chunk-QN33PNHL-DcFSm8MN.js";import"./index-BtH6urND.js";import"./_commonjsHelpers-CqkleIqs.js";import"./purify.es-HRjpPm7y.js";var f={parser:e,get db(){return new t},renderer:s,styles:a,init:i(r=>{r.class||(r.class={}),r.class.arrowMarkerAbsolute=r.arrowMarkerAbsolute},"init")};export{f as diagram};
