import{p as H,a as r,j as i,i as h,l as e,y as b,g as u,s as M,t as _,q as z,e as v,h as A,c as B,k as f,D as E}from"./index-BtH6urND.js";import F from"./Markdown-BDB-IX6n.js";import"./purify.es-HRjpPm7y.js";import"./katex-XbL3y5x-.js";/* empty css                  */var G=i('<button class="callout-header svelte-y8yhe6" type="button"><span class="callout-title"> </span> <span class="callout-toggle"> </span></button>'),I=i('<div class="callout-header svelte-y8yhe6"><span class="callout-title"> </span></div>'),J=i('<div class="callout-body svelte-y8yhe6"><!></div>'),K=i("<div><!> <!></div>");function R(g,s){H(s,!0);let k=r(s,"content",3,""),n=r(s,"type",3,"info"),y=r(s,"title",3,void 0),x=r(s,"collapsible",3,!1),o=M(!0);var c=K(),m=e(c);{var C=a=>{var t=G();t.__click=()=>B(o,!u(o));var l=e(t),p=e(l),d=b(l,2),q=e(d);_(D=>{f(p,D),f(q,u(o)?"−":"+")},[()=>y()||n().toUpperCase()]),v(a,t)},w=a=>{var t=I(),l=e(t),p=e(l);_(d=>f(p,d),[()=>y()||n().toUpperCase()]),v(a,t)};h(m,a=>{x()?a(C):a(w,!1)})}var U=b(m,2);{var j=a=>{var t=J(),l=e(t);F(l,{get content(){return k()},allowHtml:!1,padded:!1}),v(a,t)};h(U,a=>{u(o)&&a(j)})}_(()=>z(c,1,`callout ${n()}`,"svelte-y8yhe6")),v(g,c),A()}E(["click"]);export{R as default};
