.mermaid.svelte-18katb3{padding:1rem;overflow-x:auto;min-width:0}.mermaid.svelte-18katb3 svg{max-width:100%;height:auto}.mermaid.svelte-18katb3 foreignObject>div{max-width:none!important;overflow:visible!important}.mermaid.svelte-18katb3 .nodeLabel,.mermaid.svelte-18katb3 .edgeLabel,.mermaid.svelte-18katb3 .label{white-space:nowrap;overflow:visible}.mermaid.svelte-18katb3 .er.entityBox{overflow:visible}.renderer-error.svelte-18katb3{padding:.75rem 1rem;background:#fef2f2;border:1px solid #fecaca;border-radius:.5rem;color:#dc2626}
