import{a as t,j as u,t as c,w as i,l as n,r as v,e as w}from"./index-BtH6urND.js";var o=u('<div class="embed-wrapper svelte-spagmd"><iframe class="svelte-spagmd"></iframe></div>');function p(r,e){let s=t(e,"url",3,""),d=t(e,"title",3,void 0),m=t(e,"width",3,"100%"),h=t(e,"height",3,"400px"),f=t(e,"allowFullscreen",3,!0);var a=o(),l=n(a);c(()=>{i(l,"src",s()),i(l,"title",d()),v(l,`width: ${m()}; height: ${h()};`),l.allowFullscreen=f()}),w(r,a)}export{p as default};
